@@ -560,3 +560,67 @@ gdu_application_get_local_jobs_for_object (GduApplication *application,
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
+
+gboolean
+gdu_application_has_running_job (GduApplication *application,
+                                 UDisksObject   *object)
+{
+  UDisksClient *client;
+  GList *l;
+  GList *jobs = NULL;
+  GList *objects_to_check = NULL;
+  gboolean ret = FALSE;
+
+  client = gdu_application_get_client (application);
+  objects_to_check = gdu_utils_get_all_contained_objects (client, object);
+  objects_to_check = g_list_prepend (objects_to_check, g_object_ref (object));
+
+  for (l = objects_to_check; l != NULL; l = l->next)
+    {
+      UDisksObject *object_iter = UDISKS_OBJECT (l->data);
+      UDisksEncrypted *encrypted_for_object;
+
+      jobs = udisks_client_get_jobs_for_object (client, object_iter);
+      if (jobs != NULL)
+        {
+          ret = TRUE;
+          break;
+        }
+
+      jobs = gdu_application_get_local_jobs_for_object (application, object_iter);
+      if (jobs != NULL)
+        {
+          ret = TRUE;
+          break;
+        }
+
+      encrypted_for_object = udisks_object_peek_encrypted (object_iter);
+      if (encrypted_for_object != NULL)
+        {
+          UDisksBlock *block_for_object;
+          UDisksBlock *cleartext;
+
+          block_for_object = udisks_object_peek_block (object_iter);
+          cleartext = udisks_client_get_cleartext_block (client, block_for_object);
+          if (cleartext != NULL)
+            {
+              UDisksObject *cleartext_object;
+
+              cleartext_object = (UDisksObject *) g_dbus_interface_get_object (G_DBUS_INTERFACE (cleartext));
+              g_object_unref (cleartext);
+
+              ret = gdu_application_has_running_job (application, cleartext_object);
+              if (ret)
+                break;
+            }
+        }
+
+    }
+
+  g_list_foreach (jobs, (GFunc) g_object_unref, NULL);
+  g_list_free (jobs);
+  g_list_free_full (objects_to_check, g_object_unref);
+
+  return ret;
+}
+
